package org.prlab.idic.privacypolicy;

import android.bluetooth.*;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Switch;
import android.widget.TextView;
import org.prlab.idic.privacypolicy.adapter.MessageRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.ble.BLECustomScript;
import org.prlab.idic.privacypolicy.pojo.Message;
import org.prlab.idic.privacypolicy.storage.PrivacyPolicyStorageImpl;
import org.prlab.idic.privacypolicy.tool.BaseUtils;

import java.util.LinkedList;
import java.util.List;

import static android.bluetooth.BluetoothGatt.GATT_SUCCESS;

public class DeviceOperatorActivity extends AppCompatActivity {

    private static final String TAG = DeviceOperatorActivity.class.getSimpleName();
    private String PRIVACY_POLICY_ACCEPT_UUID;

    private PrivacyPolicyStorageImpl mPolicyStorage;
    private String mTargetDeviceAddress;
    private List<Message> messages = new LinkedList<>();

    private TextView mMessageBarTextView;
    private EditText mTemperatureEditText;
    private Button mTemperatureButton;
    private Switch mAirPurifierSwitch;
    private RecyclerView mMessageRecyclerView;
    private RecyclerView.Adapter mMessageRecyclerViewAdapter;

    private BluetoothManager mBluetoothManager;
    private BluetoothAdapter mBluetoothAdapter;
    private BluetoothGatt mBluetoothGatt;
    private BroadcastReceiver mPairRequestReceiver;
    private BroadcastReceiver mBondStateReceiver;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_device_operator);

        mTargetDeviceAddress = getIntent().getStringExtra(getString(R.string.intent_target_address_key));
        mPolicyStorage = new PrivacyPolicyStorageImpl(getContentResolver(), mTargetDeviceAddress);
        PRIVACY_POLICY_ACCEPT_UUID = getString(R.string.uuid_accept_privacy_policy);

        initView();
        initBluetooth();
    }

    private void initView() {
        mMessageBarTextView = (TextView) findViewById(R.id.text_operation_message_bar);
        mTemperatureEditText = (EditText) findViewById(R.id.);
        mMessageRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_operation_info);
        mMessageRecyclerViewAdapter = new MessageRecyclerViewAdapter(getResources(), messages);
        mMessageRecyclerView.setAdapter(mMessageRecyclerViewAdapter);
    }

    private void initBluetooth() {
        mBluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        if (mBluetoothManager != null) {
            mBluetoothAdapter = mBluetoothManager.getAdapter();
        }
    }

    private BLECustomScript mBLECustomScript = new BLECustomScript() {

        @Override
        public void onDiscoverServiceCompleted(BluetoothGatt gatt) {
            for (BluetoothGattService service : gatt.getServices()) {
                List<String> layerUUIDs = new LinkedList<>();
                layerUUIDs.add(String.valueOf(service.getUuid()));
                //Write service layer selection
                int selection = mPolicyStorage.get(layerUUIDs);
                if (selection != -1) {
                    for (BluetoothGattCharacteristic characteristic : service.getCharacteristics()) {
                        if (PRIVACY_POLICY_ACCEPT_UUID.equals(characteristic.getUuid().toString())) {
                            characteristic.setValue(BaseUtils.intArrray2ByteArray(selection));
                            gatt.writeCharacteristic(characteristic);
                        }
                    }
                }
                for (BluetoothGattCharacteristic characteristic : service.getCharacteristics()) {
                    layerUUIDs.add(String.valueOf(characteristic.getUuid()));
                    //Write characteristic layer selection
                    selection = mPolicyStorage.get(layerUUIDs);
                    if (selection != -1) {
                        for (BluetoothGattDescriptor descriptor : characteristic.getDescriptors()) {
                            if (PRIVACY_POLICY_ACCEPT_UUID.equals(descriptor.getUuid().toString())) {
                                descriptor.setValue(BaseUtils.intArrray2ByteArray(selection));
                                gatt.writeDescriptor(descriptor);
                            }
                        }
                    }
                    layerUUIDs.remove(layerUUIDs.size() - 1);
                }
            }
        }

        @Override
        public void onWriteCompleted(BluetoothGatt gatt, int status, byte[] values) {

        }

        @Override
        public void onReadCompleted(BluetoothGatt gatt, int status, byte[] values) {

        }

        @Override
        public void onPairRequestReceived(BluetoothDevice device, int pin) {

        }

        @Override
        public void onPaired(BluetoothDevice device) {

        }

        @Override
        public void onUnpaired(BluetoothDevice device) {

        }

    };

    private BluetoothGattCallback mBluetoothGattCallback = new BluetoothGattCallback() {

        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            String mac = gatt.getDevice().getAddress();
            if (status == GATT_SUCCESS) {
                if (newState == BluetoothGatt.STATE_CONNECTED) {
                    gatt.discoverServices();
                } else if (newState == BluetoothGatt.STATE_DISCONNECTED) {
                    mBluetoothGatt.close();
                    mBluetoothGatt = null;
                }
            }
            //重連機制，但好像只會一直bind住。
            else if (status == 133 && newState == BluetoothGatt.STATE_DISCONNECTED) {
                mBluetoothGatt.close();
                connect();
            }
        }

        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == GATT_SUCCESS) {
                mBLECustomScript.onDiscoverServiceCompleted(gatt);
            }
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            if ((status & (BluetoothGatt.GATT_INSUFFICIENT_AUTHENTICATION | BluetoothGatt.GATT_INSUFFICIENT_ENCRYPTION)) != 0) {
                boolean isSuccessful = gatt.getDevice().createBond();
                Log.i(TAG, "Create Bond is " + isSuccessful);
            } else {
                mBLECustomScript.onReadCompleted(mBluetoothGatt, status, characteristic.getValue());
            }
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            mBLECustomScript.onWriteCompleted(mBluetoothGatt, status, characteristic.getValue());
        }

    };

    private void registerPairRequestReceiver() {
        IntentFilter intentFilter = new IntentFilter(BluetoothDevice.ACTION_PAIRING_REQUEST);
        BroadcastReceiver receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if (intent.getAction().equals(BluetoothDevice.ACTION_PAIRING_REQUEST)) {
                    Log.d(TAG, "PIN " + intent.getIntExtra(BluetoothDevice.EXTRA_PAIRING_KEY, 0));
                    BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                    int pin = intent.getIntExtra(BluetoothDevice.EXTRA_PAIRING_KEY, 0);
                    mBLECustomScript.onPairRequestReceived(device, pin);
                }
            }
        };
        registerReceiver(receiver, intentFilter);
        mPairRequestReceiver = receiver;
    }

    private void unregisterPairRequestReceiver() {
        if (mPairRequestReceiver != null) {
            unregisterReceiver(mPairRequestReceiver);
            mPairRequestReceiver = null;
        }
    }

    private void registerBondStateReceiver() {
//        mMessageCallback.displayMessage("Pair", "Ready to pair " + mTargetAddress, true);
        IntentFilter intentFilter = new IntentFilter(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
        BroadcastReceiver receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                String action = intent.getAction();
                if (BluetoothDevice.ACTION_BOND_STATE_CHANGED.equals(action)) {
                    final int state = intent.getIntExtra(BluetoothDevice.EXTRA_BOND_STATE, BluetoothDevice.ERROR);
                    final int preState = intent.getIntExtra(BluetoothDevice.EXTRA_PREVIOUS_BOND_STATE, BluetoothDevice.ERROR);
                    Log.i(TAG, "Bond [" + preState + " to " + state + "].");
                    BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                    if (state == BluetoothDevice.BOND_BONDED && preState == BluetoothDevice.BOND_BONDING) {
                        mBLECustomScript.onPaired(device);
                    } else if (state == BluetoothDevice.BOND_NONE && preState == BluetoothDevice.BOND_BONDED) {
                        mBLECustomScript.onUnpaired(device);
                    }
                }

            }
        };
        registerReceiver(receiver, intentFilter);
        mBondStateReceiver = receiver;
    }

    private void unregisterBondStateReceiver() {
        if (mBondStateReceiver != null) {
            unregisterReceiver(mBondStateReceiver);
            mBondStateReceiver = null;
        }
    }

    private boolean connect() {
        if (mBluetoothAdapter == null) return false;
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(mTargetDeviceAddress);

        int connectionState = mBluetoothManager.getConnectionState(device, BluetoothProfile.GATT);
        Log.i(TAG, "Device connection status is " + connectionState + ".");

        if (device != null && connectionState == BluetoothProfile.STATE_DISCONNECTED) {
            mBluetoothGatt = device.connectGatt(this, false, mBluetoothGattCallback);
            return mBluetoothGatt != null;
        } else {
            return false;
        }
    }


    private void disconnect() {
        if (mBluetoothGatt != null) {
            int connectionState = mBluetoothManager.getConnectionState(mBluetoothGatt.getDevice(), BluetoothProfile.GATT);
            Log.i(TAG, "Device connection status is " + connectionState + ".");
            if (connectionState == BluetoothProfile.STATE_CONNECTED) {
                boolean isBonded = mBluetoothAdapter.getBondedDevices().contains(mBluetoothGatt.getDevice());
                Log.i(TAG, "Device is already bonded. :" + isBonded);
//                try {
//                    Method method = mBluetoothGatt.getDevice().getClass().getMethod("removeBond", (Class[]) null);
//                    method.invoke(mBluetoothGatt.getDevice(), (Object[]) null);
//                } catch (NoSuchMethodException | InvocationTargetException | IllegalAccessException e) {
//                    e.printStackTrace();
//                }
                mBluetoothGatt.disconnect();
            }
        }
    }
}
